import { EventEmitter2 } from "eventemitter2";
import { clearLine, clearScreenDown, cursorTo, moveCursor } from './utils';
declare const kLineObjectStream: unique symbol;
declare const kQuestionCancel: unique symbol;
/**
 * Creates a new `readline.Interface` instance.
 * @param {Readable | {
 *   input: Readable;
 *   output: Writable;
 *   completer?: Function;
 *   terminal?: boolean;
 *   history?: string[];
 *   historySize?: number;
 *   removeHistoryDuplicates?: boolean;
 *   prompt?: string;
 *   crlfDelay?: number;
 *   escapeCodeTimeout?: number;
 *   tabSize?: number;
 *   signal?: AbortSignal;
 *   }} input
 * @param {Writable} [output]
 * @param {Function} [completer]
 * @param {boolean} [terminal]
 * @returns {Interface}
 */
declare function createInterface(input: any, output?: any, completer?: any, terminal?: any): Interface;
declare class Interface extends EventEmitter2 {
    private input;
    private output;
    private completer;
    private _sawReturnAt;
    private isCompletionEnabled;
    private _sawKeyPress;
    private _previousKey?;
    private escapeCodeTimeout;
    private tabSize;
    private closed;
    private paused;
    private terminal;
    private history;
    private historySize;
    private removeHistoryDuplicates;
    private crlfDelay;
    private _oldPrompt;
    private _prompt;
    private signal;
    private _decoder?;
    private _questionCallback;
    private prevRows?;
    private cursor;
    private historyIndex;
    private line;
    private _line_buffer;
    private [kQuestionCancel];
    private [kLineObjectStream];
    constructor(input: any, output: any, completer: any, terminal: any);
    get columns(): any;
    private onerror;
    private ondata;
    private onend;
    private ontermend;
    private onkeypress;
    private onresize;
    /**
     * Sets the prompt written to the output.
     * @param {string} prompt
     * @returns {void}
     */
    setPrompt(prompt: any): void;
    /**
     * Returns the current prompt used by `rl.prompt()`.
     * @returns {string}
     */
    getPrompt(): string;
    private _setRawMode;
    /**
     * Writes the configured `prompt` to a new line in `output`.
     * @param {boolean} [preserveCursor]
     * @returns {void}
     */
    prompt(preserveCursor?: any): void;
    /**
     * Displays `query` by writing it to the `output`.
     * @param {string} query
     * @param {{ signal?: AbortSignal; }} [options]
     * @param {Function} cb
     * @returns {void}
     */
    question(query: any, options: any, cb: any): void;
    private _questionCancel;
    private _onLine;
    private _writeToOutput;
    private _addHistory;
    private _refreshLine;
    /**
     * Closes the `readline.Interface` instance.
     * @returns {void}
     */
    close(): void;
    /**
     * Pauses the `input` stream.
     * @returns {void | Interface}
     */
    pause(): this | undefined;
    /**
     * Resumes the `input` stream if paused.
     * @returns {void | Interface}
     */
    resume(): this | undefined;
    /**
     * Writes either `data` or a `key` sequence identified by
     * `key` to the `output`.
     * @param {string} d
     * @param {{
     *   ctrl?: boolean;
     *   meta?: boolean;
     *   shift?: boolean;
     *   name?: string;
     *   }} [key]
     * @returns {void}
     */
    write(d?: any, key?: any): void;
    private _normalWrite;
    private _insertString;
    private _tabComplete;
    private _wordLeft;
    private _wordRight;
    private _deleteLeft;
    private _deleteRight;
    private _deleteWordLeft;
    private _deleteWordRight;
    private _deleteLineLeft;
    private _deleteLineRight;
    clearLine(): void;
    private _line;
    private _historyNext;
    private _historyPrev;
    private _getDisplayPos;
    /**
      * Returns the real position of the cursor in relation
      * to the input prompt + string.
      * @returns {{
      *   rows: number;
      *   cols: number;
      *   }}
      */
    getCursorPos(): {
        cols: number;
        rows: number;
    };
    private _getCursorPos;
    private _moveCursor;
    private _ttyWriteDumb;
    private _ttyWrite;
}
export { Interface, clearLine, createInterface, clearScreenDown, cursorTo, moveCursor };
